/*
 * Copyright (c) 2007, intarsys consulting GmbH
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * - Neither the name of intarsys nor the names of its contributors may be used
 *   to endorse or promote products derived from this software without specific
 *   prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package de.intarsys.tools.expression;

import java.util.HashMap;
import java.util.Map;

import de.intarsys.tools.functor.IArgs;

/**
 * An {@link IStringEvaluator} that looks up a variable in a simple Map.
 * <p>
 * Only named (associative) lookup is supported.
 * 
 */
public class MapResolver extends ContainerResolver {
	/**
	 * The Map where we can lookup a variable.
	 */
	final private Map variables;

	final private boolean strict;

	public MapResolver() {
		this(new HashMap(), false);
	}

	public MapResolver(boolean strict) {
		this(new HashMap(), strict);
	}

	public MapResolver(Map variables) {
		this(variables, false);
	}

	public MapResolver(Map variables, boolean strict) {
		super();
		this.variables = variables;
		this.strict = strict;
	}

	@Override
	protected Object basicEvaluate(String expression, IArgs args)
			throws EvaluationException {
		if (variables.containsKey(expression)) {
			return variables.get(expression);
		}
		if (strict) {
			// map resolver MUST indicate lookup failure to provide for nested
			// dynamic lookup
			throw new EvaluationException("can't evaluate '" + expression + "'"); //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			return null;
		}
	}

	public void put(String key, Object value) {
		variables.put(key, value);
	}
}
